/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.siemens_enterprise.openstagepush.data;


import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.Collection;
import java.util.NavigableMap;
import java.util.Set;
import java.util.TreeMap;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.servlet.ServletContext;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.Transformer;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 *
 * @author atw10t87
 */
public class DataService {

    private static final Log LOGGER = LogFactory.getLog("DataService");
    private static final String PHONES_FILE = "/phones/phones.xml";
    private static final String PHONE_TAG = "phone";
    private static final String PHONES_TAG = "phones";
    private static final String PHONE_ADDRESS_ATTR = "address";

    private static DataService dataService = null;
    private ServletContext servletContext = null;
    private NavigableMap<String, Phone> phones = null;
    private NavigableMap<String, XmlFile> xmlFiles = null;
    private final Object phonesLock = new Object();
    private final Object xmlFilesLock = new Object();

    private DataService(ServletContext ctx) {
        this.servletContext = ctx;
        phones = new TreeMap<String, Phone>();
        xmlFiles = new TreeMap<String, XmlFile>();
        readData();
    }

    public Collection<Phone> getPhones() {
        synchronized (phonesLock) {
            return phones.values();
        }
    }

    public Collection<XmlFile> getXmlFiles() {
        synchronized (xmlFilesLock) {
            return xmlFiles.values();
        }
    }

    public void addPhone(Phone phone) {
        synchronized (phonesLock) {
            phones.put(phone.getPhoneAddress(), phone);
            savePhones();
        }
    }

    public void deletePhone(String phoneId) {
        synchronized (phonesLock) {
            phones.remove(phoneId);
            savePhones();
        }
    }

    private void readData() {
        readPhones();
        readFiles();
    }

    private void readPhones() {
        InputStream in = null;

        in = servletContext.getResourceAsStream(PHONES_FILE);

        if (in != null) {
                //parse XML file
                DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
                try {
                    DocumentBuilder docBuilder = factory.newDocumentBuilder();
                    Document phonesDoc = docBuilder.parse(in);

                    NodeList phonesList = phonesDoc.getElementsByTagName(PHONE_TAG);
                    for (int i = 0; i < phonesList.getLength(); i++) {
                        Element elPhone = (Element) phonesList.item(i);
                        String phoneAddress = elPhone.getAttribute(PHONE_ADDRESS_ATTR);
                        Phone phone = new Phone(phoneAddress);
                        phones.put(phoneAddress, phone);
                    }

                } catch (SAXException ex) {
                    LOGGER.error("Phones.xml parser's error", ex);
                } catch (IOException ex) {
                    LOGGER.error("Phones.xml parser's error", ex);
                } catch (ParserConfigurationException ex) {
                    LOGGER.error("Phones.xml parser's error", ex);
                }


            }

    }

    private void savePhones() {
        String realPath = servletContext.getRealPath(PHONES_FILE);

        if (realPath == null) {
            LOGGER.fatal("Could not get real path for " + PHONES_FILE + ". Are we running from WAR file?");
            return;
        }

        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            DocumentBuilder docBuilder = factory.newDocumentBuilder();
            Document doc = docBuilder.newDocument();
            Element phonesElement = doc.createElement(PHONES_TAG);
            doc.appendChild(phonesElement);
            Set<String> phonesKeys = phones.keySet();
            synchronized (phonesLock) {
                for (String phoneKey : phonesKeys) {
                    if (phoneKey != null) {
                        Element phoneElement = doc.createElement(PHONE_TAG);
                        phoneElement.setAttribute(PHONE_ADDRESS_ATTR, phoneKey);
                        phonesElement.appendChild(phoneElement);
                    }
                }
            }

            TransformerFactory tFactory = TransformerFactory.newInstance();
            Transformer transformer = tFactory.newTransformer();

            DataService.setupTransformer(transformer);

            DOMSource source = new DOMSource(doc);
            StreamResult result = new StreamResult(new File(realPath));

            transformer.transform(source, result);

        } catch (TransformerConfigurationException ex) {
            Logger.getLogger(DataService.class.getName()).log(Level.SEVERE, null, ex);
        } catch (TransformerException ex) {
            Logger.getLogger(DataService.class.getName()).log(Level.SEVERE, null, ex);
        } catch (ParserConfigurationException ex) {
            LOGGER.error("Phones.xml parser's error", ex);
        }

        LOGGER.debug("Attempting to save config file: " + realPath + ", result -> OK");

    }

    private void readFiles() {
        String xmlDirPath = servletContext.getRealPath("/xml/");

        File xmlDir = new File(xmlDirPath);

        for (String file:xmlDir.list()) {
            XmlFile xmlFile = new XmlFile(file);
            xmlFiles.put(file, xmlFile);
        }

    }

    public static void setupTransformer(Transformer t) {
        t.setOutputProperty(OutputKeys.ENCODING, "UTF-8");
        t.setOutputProperty(OutputKeys.INDENT, "yes");
        t.setOutputProperty(OutputKeys.METHOD, "xml");
        t.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "2");
    }

    public static DataService getDataService(ServletContext ctx) {
        synchronized (DataService.class) {
            if (dataService == null) {
                dataService = new DataService(ctx);
            }

        }

        return dataService;

    }
}
